*-- File: Ini.Cc

CLASS Ini(cIniFile)
*-----------------------------------------------------------------------
*-- Description:
*
*   CLASS Ini wraps GetPrivateProfileString and
*   WritePrivateProfileString, the Windows API functions most commonly
*   used to create, read, and write application-specific .INI files.
*
*-- Programmers:
*
*   Keith Chuvala (CIS: 71333,2654)
*   Bowen Moursund (CIS: 72662,436)
*   Kenneth Chan (CIS: 104471,3576)
*
*-- History:
*
*   January 1996: Original publication in dBASE Advisor.
*   08/13/1996:   Minor revisions and addition of GetSection, Flush,
*                 DeleteSection, DeleteEntry, and supporting methods.
*
*-- Instantiation:
*
*   set procedure to Ini.Cc additive
*   oMyINI = new INI([cIniFile])
*
*   Typical usage is to specify the full path to the .INI file via the
*   cIniFile parameter. Examples:
*
*       oMyIni = new Ini(set("directory")+"\MyApp.Ini")
*       oMyIni = new Ini(home()+"MyApp.Ini")
*       oMyIni = new Ini(_dbwinhome+"MyApp.Ini")
*
*   The cIniFile parameter is optional. If cIniFile is not specified,
*   the return of program() will be used as the file name. If the path
*   is not specified, the VdB home() directory will be used. You may
*   set the Inifile property directly to a name with no path if you
*   wish to use the Windows home directory.

*-- Create/Write:
*
*   oMyIni.SetValue(cSection,cEntry,cValue)
*
*   If the .INI file does not exist, it will be created. The cSection
*   section will be added to the .INI file if it does not exist. An
*   entry in the form of Entry=Value will be added to the cSection
*   section. Example:
*
*       oMyIni.SetValue("Window","Top",str(form.Top))
*
*-- Read Entry:
*
*   oMyIni.GetValue(cSection,cEntry)
*
*   This method returns the value of the specified entry of the
*   specified section as well as storing the value to the Value
*   property of the Ini object. The return value will be blank
*   if the entry is not found, the section does not exist, or
*   if a problem occurs reading the .INI file, UNLESS you set
*   the Default property. Example:
*
*       oMyIni.Default = "1"
*       oMyIni.GetValue("Window","Top")
*
*-- Read Section:
*
*   oMyIni.GetSection(cSection)
*
*   This method returns a reference to an array object containing all
*   the entries of a section. Each element of the array will be in the
*   form Entry=Value. The array reference will also be stored to the
*   SectionArray property of the Ini object. The array will contain only
*   a single element, a null string, if the section does not exist or if
*   a problem occurs reading the .INI file.
*
*-- Parse Section Array:
*
*   oMyIni.SectionArrayParse2Arrays([cDelimiter])
*
*   This method parses the SectionArray array into two parallel arrays,
*   one containing the Entries, the other the matching Values.
*   References to the arrays are stored to the EntriesArray and
*   ValuesArray properties of the Ini object. The method returns a
*   reference to yet another array containing two elements: element 1 is
*   a reference to the EntriesArray array, element 2 is a reference to
*   the ValuesArray array. The array referenced by the return will be
*   empty if the SectionArray array is empty.
*
*   The cDelimiter parameter is optional. Some entries may have multiple
*   Values, delimited by a certain character, usually the comma. If
*   cDelimiter is specified and there actually are multiple Values
*   delimited by the specified character, the ValuesArray array will be
*   2 dimensional. The number of columns will be the maximum number of
*   Values encountered for any single Entry.
*
*-- Delete Section:
*
*   oIni.DeleteSection(cSection)
*
*   This method deletes the specified section from the .INI file. It
*   returns 0 if unsuccessful.
*
*-- Delete Entry:
*
*   oIni.DeleteEntry(cSection,cEntry)
*
*   This method deletes the specified entry of the specified section
*   from the .INI file. It returns 0 if unsuccessful.
*
*-- Flush Updates:
*
*   oIni.Flush()
*
*   Flushes any updates to the .INI file to disk.
*
*-- New .INI file:
*
*   oMyIni.SetIniFile(cIniFile)
*
*   Notes are the same as for Instantiation, above.
*
*-- Release:
*
*   oMyIni.Release()
*
*   Releases the Ini object.
*
*-- Properties:
*
*   IniFile: File name of the .INI file
*   Section: Last read/written section
*   Entry: Last read/written entry
*   Default: Default value for the GetValue method, normally blank
*   SectionArray: Last read 1D array of Entries of a Section in the
*                 form Entry=Value
*   EntriesArray: Last parsed Entries from SectionArray
*   ValuesArray: Last parsed Values from SectionArray
*-----------------------------------------------------------------------

  #define INI_RETSIZE 320
  #define INI_SECTION_RETSIZE 32000
  #ifndef NULL
      #define NULL 0
  #endif
  this.Section = ""
  this.Entry = ""
  this.Inifile = ""
  this.Value = ""
  this.Default = ""
  this.SectionArray = new array()
  this.SectionArray.add("")
  this.EntriesArray = new array()
  this.EntriesArray.add("")
  this.ValuesArray = new array()
  this.ValuesArray.add("")
  class::SetIniFile(cIniFile)
  if type("GetPrivateProfileString") <> "FP"
      extern CINT GetPrivateProfileString( CSTRING,;
                                           CSTRING,;
                                           CSTRING,;
                                           CPTR,;
                                           CINT,;
                                           CSTRING ) KRNL386.EXE
  endif
  if type("WritePrivateProfileString") <> "FP"
      extern CINT WritePrivateProfileString( CSTRING,;
                                             CSTRING,;
                                             CSTRING,;
                                             CSTRING ) KRNL386.EXE
  endif

  PROCEDURE GetValue
     parameter cSection, cEntry
     this.value = space(INI_RETSIZE)
     if type("cSection") == "C" .and. type("cEntry") == "C"
        this.Section = cSection
        this.Entry = cEntry
        GetPrivateprofileString(this.Section, this.Entry, this.Default,;
                              this.Value, INI_RETSIZE, this.IniFile)
        this.Value = trim(this.Value)
        if right(this.Value,1) = chr(0)
           this.Value = ltrim(left(" " + this.value, len(this.value)))
        endif
    endif
  RETURN trim(this.Value)

  PROCEDURE SetValue
    parameter cSection, cEntry, cNewValue
    local nRetVal
    nRetVal = 0
    if type("cSection") == "C" .and. type("cEntry") == "C" .and.;
       type("cNewValue") == "C"
       this.Section = cSection
       this.Entry = cEntry
       this.Value = ltrim(trim(cNewValue))
       nRetVal = WritePrivateProfileString(this.Section, this.Entry,;
                                         this.Value, this.IniFile)
    endif
  RETURN nRetVal

  PROCEDURE SetIniFile
  parameter cIniFile
  if type("cIniFile") <> "C" && no file name specified
      *-- use program name
      this.IniFile = left(program(),8)+".INI"
  else
      this.IniFile = ltrim(trim(cIniFile))
  endif
  if .not. ("\" $ this.IniFile)  && if no path specified
      *-- use VdB home directory
      this.IniFile = home() + this.IniFile
  endif
  if right(upper(this.IniFile),4) <> ".INI"
      *-- ensure correct extension
      local nPos
      nPos = at(".", this.IniFile)
      if nPos > 0
          this.IniFile = stuff(this.IniFile, nPos, 4, ".INI")
      else
          this.IniFile = this.IniFile + ".INI"
      endif
  endif

  PROCEDURE GetSection
  parameter cSection
  this.SectionArray = new array()
  if type("cSection") == "C"
      this.Section = cSection
      local nBytes, cBuffer
      cBuffer = space(INI_SECTION_RETSIZE)
      nBytes = GetPrivateProfileString(cSection, NULL, this.Default,;
                                       cBuffer, INI_SECTION_RETSIZE,;
                                       this.IniFile )
      if nBytes = 0
          this.SectionArray.add("")
      else
          local cValueBuffer, nPosition, cValue, cEntry
          cValueBuffer = space(INI_RETSIZE)
          nPosition = 1
          do while nPosition < nBytes
              cEntry = substr(cBuffer, nPosition)
              cEntry = left(cEntry, at(chr(0), cEntry) - 1)
              nPosition = nPosition + len(cEntry) + 1
              GetPrivateProfileString(cSection, cEntry, ;
                                      cEntry + " not found",;
                                      cValueBuffer, INI_RETSIZE,;
                                      this.IniFile)
              cValue = left(cValueBuffer, at(chr(0), cValueBuffer) - 1)
              this.SectionArray.add(cEntry + "=" + cValue)
          enddo
      endif
  else
      this.SectionArray.add("")
  endif
  RETURN this.SectionArray

  PROCEDURE Flush
  WritePrivateProfileString(0, 0, 0, this.IniFile)

  PROCEDURE DeleteEntry
  parameter cSection, cEntry
  local nRetVal
  nRetVal = 0
  if type("cSection") == "C" .and. type("cEntry") == "C"
      nRetVal = WritePrivateProfileString(cSection, cEntry, 0, this.IniFile)
  endif
  RETURN nRetVal

  PROCEDURE DeleteSection
  parameter cSection
  local nRetVal
  nRetVal = 0
  if type("cSection") == "C"
      nRetVal = WritePrivateProfileString(cSection, 0, 0, this.IniFile)
  endif
  RETURN nRetVal

  PROCEDURE SectionArrayParse2Arrays
  parameters cDelimiter
  local aArrays, aEntries, aValues, nRows, nCols, n, cString, nLen,;
        nPosition, nCol, cSubStr
  aArrays = new array()
  nRows= alen(this.SectionArray,1)
  if nRows > 0
      this.EntriesArray = new array()
      nCols = 1
      for n = 1 to nRows
          this.EntriesArray.add(left(this.SectionArray[n],;
                                at("=",this.SectionArray[n])-1))
          if type("cDelimiter") = "C"
              nCols = max(nCols, class::CharCount(cDelimiter,;
                          this.SectionArray[n]) + 1)
          endif
      endfor
      aArrays.add(this.EntriesArray)
      this.ValuesArray = new array(nRows, nCols)
      for n = 1 to nRows
          cString = substr(this.SectionArray[n],at("=",;
                           this.SectionArray[n])+1)
          if type("cDelimiter") = "C"
              nLen = len(cString)
              nPosition = 1
              nCol = 1
              do while nPosition < nLen .and. nCol <= nCols
                  cSubStr = substr(cString, nPosition)
                  if nCol < nCols
                      cSubStr = left(cSubStr, at(cDelimiter, cSubStr) - 1)
                  endif
                  nPosition = nPosition + len(cSubStr) + 1
                  this.ValuesArray[n,nCol] = cSubStr
                  nCol = nCol + 1
              enddo
          else
              this.ValuesArray[n,1] = cString
          endif
      endfor
      aArrays.add(this.ValuesArray)
  endif
  RETURN aArrays

  PROCEDURE Release
  release object this

  PROCEDURE CharCount(cChar, cStr)
  local cString, nCnt
  cString = cStr
  nCnt = 0
  do while cChar $ cString
      nCnt = nCnt + 1
      cString = substr( cString, at(cChar, cString)+1 )
  enddo
  RETURN nCnt

ENDCLASS
*-- End of Class: Ini.Cc

*-----------------------------------------------------------------------
*-- End of File: INI.CC
*-----------------------------------------------------------------------