*-- File: TEXT.CC

*----------------------------------------------------------------------
*-- Description:
*
*      This UI file is designed to give some optional text objects
*      which can be used either 'as is' or you can even subclass them.
*
*-- Programmers: 
*
*      Ken Mayer
*      Romain Strieff (CIS: 71333,2147)
*
*-- History: 
*
*      12/03/1996 -- First version of this custom class file.
*
*-- Instantiation: 
*
*      set PROCEDURE to text.cc additive
*      * create/modify a form, place an instance of one of
*      * these entryfields on the form as a normal entryfield ...
*
*-- Classes:
*      More details on each of these is contained in the commentary for
*      each class ...
*
*--    BlinkText
*        A blinking text object ...
*
*--    HandText
*        A Text object showing a 'HAND' cursor 
*
*--    ClockText
*        A Text object that uses a timer to display the current time
*        on a form.
*
*--    VerticalText
*        A text control that displays vertically, rather than
*        horizontally -- displays text from bottom toward the top.
*        (A working example of this is shown by running the 
*         form: VERTTEXT.WFM )
*
*        NOTE: While this is really a paintbox control, it's included 
*              here, as what appears on the form is text ...
*
*-- Dependencies:
*
*      HAND.DLL  (used by HandText)
*
*-- To see an example of each of these working, run the form:
*      TEXT.WFM
*
*----------------------------------------------------------------------

CLASS BlinkText(FormObj,Name) OF TEXT(FormObj,Name) Custom
   *-------------------------------------------------------------------
   *-- Description:
   *
   *     Custom TEXT Object blinking twice per second to draw attention 
   *     of the use to special texts etc. This version uses a single 
   *     timer for all blinking objects on a single form. 
   *
   *-- Programmers: 
   *
   *      Romain Strieff (CIS: 71333,2147)
   *
   *-- History: 
   *
   *      05/13/1996 -- Original
   *      11/28/1996 -- Very minor modifications for the dUFLP library.
   *
   *-- Instantiation: 
   *
   *      set PROCEDURE to text.cc additive
   *      * create/modify a form, place an instance of this text object
   *      * on the form, and set the text property as usual.
   *
   *-- WARNING:
   *     In the form's OnClose code, you need to explicitly deactivate the
   *     timer object. Otherwise you will have the timer continuing to
   *     look for the codeblock it's supposed to execute ...
   *       form.BlinkTextTimer.enabled = .f.
   *----------------------------------------------------------------------

   Protect BlinkTextTimer

   *-- Attach the blink timer as custom property of the form
   *-- so we can use multiple Blinktexts with _one_ timer

   if type("Form.BlinktextTimer")="U"
      *-- only first blinktext object on a form creates the timer
      form.BlinktextTimer=new timer()
      *-- Add a custom array property to store the blinking controls
      form.BlinktextTimer.aBlinkobjects=new array()
      *-- Change this interval if you want slower/faster blinking
      form.BlinktextTimer.interval=.5
      *-- use a single visible property to assure synchronisation of 
      *-- blinking
      form.BlinktextTimer.visible=.t. 
      *--Create a reference to FORM since timer()s do not recognize it
      form.BlinktextTimer.parent=form 
      form.BlinktextTimer.ontimer={;this.visible=.not. this.visible;
                                   ;for n=1 to this.aBlinkObjects.size;
                                   ;  this.aBlinkObjects[n].visible=;
                                      this.visible;
                                   ;next}
      form.BlinktextTimer.enabled=.t.
   endif

   *-- each instance of BlinkText just adds itself to the custom array
   form.BlinktextTimer.aBlinkobjects.add(this)

ENDCLASS
*-- End of Class: BlinkText

CLASS ClockText(FormObj,Name) OF TEXT(FormObj,Name) Custom
   *-------------------------------------------------------------------
   *-- Description:
   *
   *      Custom TEXT Object showing a clock. Uses the undocumented
   *      timer object ... displays time in 12 hour format.
   *
   *-- Programmer: 
   *
   *      Romain Strieff (CIS: 71333,2147)
   *
   *-- History: 
   *
   *      11/16/1996 -- Original
   *      11/29/1996 -- Very minor modifications for the dUFLP library.
   *
   *-- Instantiation: 
   *
   *      set PROCEDURE to text.cc additive
   *      * create/modify a form, place an instance of this text object
   *      * on the form. You can change the color, font, size, etc.
   *-------------------------------------------------------------------

   this.OnOpen         = {; this.text = this.settime() }
   this.width          = 12
   this.height         =  1
   this.fontbold       = .t.
   this.timer          = new Timer()
   this.timer.interval = 1
   this.timer.oParent  = this
   this.timer.ontimer  = {;this.oParent.text = class::settime()}
   this.timer.enabled  = .t.
   
   FUNCTION SetTime
      *-- grab current time
      cTime = trim( time() )
      *-- get numeric value of 'hours'
      nHour = val( substr( cTime, 1, at( ":", cTime) -1) )
      *-- if hour is 12, we're at 'pm'
      if nHour = 12
         cSuffix = " pm"
         *-- if the hour is greater than 12, we need to
         *-- set the suffix to 'pm', but also set the 
         *-- hour to it's value - 12 (i.e., 13 is 1 pm)
         elseif nHour > 12
                cSuffix = " pm"
                nHour = nHour - 12
         *-- after midnight, before noon:
      else
         cSuffix = " am"
      endif
      *-- get everything to the right of the hour:
      cRight = substr( cTime, at(":", cTime) )
      *-- put it all together:
      cTime  = ltrim( str( nHour )) + cRight+ cSuffix
      
   RETURN cTime

ENDCLASS
*-- End of Class: ClockText

CLASS HandText(FormObj,Name) OF TEXT(FormObj,Name) Custom
   *-------------------------------------------------------------------
   *-- Description:
   *
   *      Custom TEXT Object showing a 'HAND' cursor when moving with 
   *      the mouse over it. Can be used to simulate 'hot-links' etc
   *
   *-- Programmer: 
   *
   *      Romain Strieff (CIS: 71333,2147)
   *
   *-- History: 
   *
   *      08/09/1996 -- Original
   *      11/28/1996 -- Very minor modifications for the dUFLP library.
   *
   *-- Instantiation: 
   *
   *      set PROCEDURE to text.cc additive
   *      * create/modify a form, place an instance of this text object
   *      * on the form, and set the text property as usual.
   *
   *-- Dependencies:
   *
   *      HAND.DLL -- contains a single cursor 'inspired' by the
   *      one that Netscape uses.
   *
   *-- Other Info:
   *
   *      To use this as a 'hotlink' you may want to add some
   *      code to the event: OnLeftMouseUp, and you probably
   *      want to call (before doing anything else) the 
   *      'ReleaseCapture()' function, to reset the mouse pointer.
   *
   *-- Warning: On instantiation, this class grabs the form's OnClose
   *            event, and adds to it, by storing a reference to the
   *            form's OnClose and tacking on code to release the 
   *            dll used for the hand icon. When the form is actually
   *            closed, your OnClose event is executed as well as the
   *            code needed to release the dll.
   *-------------------------------------------------------------------

   protect OnMouseMove,OnOpen
   this.Text          ="HANDTEXT"
   this.FontUnderline = .T.
   this.ColorNormal   = "R+/BtnFace"
   this.FontBold      = .F.

   PROCEDURE OnOpen
      *-- We are installing the handcursor only _once_ no matter how 
      *-- many instances of this .CC is in the form
      if type("_app.Handcursor")="U"
         if type("LoadLibrary") # "FP"
            extern CHANDLE LoadLibrary(CPTR) Kernel
         endif
         if type("FreeLibrary") # "FP"
            extern CVOID FreeLibrary(CHANDLE) Kernel
         endif
         if type("SetCursor") # "FP"
            extern CHANDLE SetCursor(CHANDLE) User
         endif
         if type("DestroyCursor") # "FP"
            extern CLOGICAL DestroyCursor(CHANDLE) User
         endif
         if type("SetCapture") # "FP"
            extern CHANDLE SetCapture(CHANDLE) User
         endif
         if type("ReleaseCapture") # "FP"
            extern CVOID   ReleaseCapture() User
         endif
         if type("LoadCursorInt") # "FP"
            extern CHANDLE LoadCursorInt(CHANDLE, CLONG) ;
                                    User From "LoadCursor"
         endif
         _app.Handcursor = LoadLibrary("HAND.DLL" + Chr(0))
         _app.hCursor    = LoadCursorInt(_app.HandCursor, 1)
      endif   

      *--Install a cursor removing call to the forms OnClose event
      if .not. empty(Form.OnClose)
         if type("Form.RemoveHandCursorsInstalled") = "U"
            form.RemoveHandCursorsOnClose = form.onclose
            form.OnClose=;
           {;form.RemovehandCursorsOnclose();CLASS::RemoveHandCursor()}
            Form.RemoveHandCursorsInstalled=.t.
         endif
      else
         form.OnClose= CLASS::RemoveHandCursor
         form.RemoveHandCursorsInstalled=.t.
      endif
   
   PROCEDURE OnMouseMove(nFlags, nCol, nRow)
      if nCol >= 0 .and. nCol <= this.Width .AND.;
         nRow >= 0 .AND. nRow <= this.Height && if cursor is over 
                                             && control
         SetCursor(_app.hCursor)
         SetCapture(this.hWND)
      else                                   && Revert to default 
                                             && cursor
         ReleaseCapture()
      endif
   
   PROCEDURE RemoveHandCursor
      if type("_app.HandCursor") # "U"
         DestroyCursor(_app.hCursor)
         FreeLibrary(_app.HandCursor)  && Must free the DLL on 
                                       && form close
         _app.HandCursor=NULL
         _app.hCursor=NULL
      endif

ENDCLASS
*-- End of Class: HandText

CLASS VerticalText(oParent, cName) of PAINTBOX(oParent, cName) Custom
   *-------------------------------------------------------------------
   *-- Description:
   *
   *      Simple Vertical text (Label) for displaying text beginnining 
   *      at the bottom-to-top (default) based on code by Jim Sare
   *
   *-- Programmer: 
   *
   *      Romain Strieff (CIS: 71333,2147)
   *
   *-- History: 
   *
   *      11/16/1996 -- Original
   *      12/03/1996 -- Very minor modifications for the dUFLP library.
   *                    Tinkered with (with suggestions from Romain)
   *                    to make this able to paint text from bottom-to-
   *                    top (default) and top-to-bottom ...
   *
   *-- Instantiation: 
   *
   *      set PROCEDURE to text.cc additive
   *      * create/modify a form, place an instance of this text object
   *      * on the form:
   *
   *      * to set text property to a value, you probably will need to
   *      * set this in the form's OnOpen, the object's OnOpen,
   *      * or an overridden open/readmodal event
   *
   *      The following is from the sample form: TEXT.WFM, the
   *      onopen event for a vertical text control (this is the one
   *      with the blue text, on the left)
   *
   *      *-- When last parameter is .T., a redraw occurs.
   *      this.fontname("Times New Roman",.f.) && set font
   *      this.drawtext("VERTICAL TEXT",.f.)   && display text in quotes
   *      this.fontunderline(.t.,.f.)          && underline it
   *      this.colortext("FF0000",.f.)         && color is blue
   *      this.fontwidth(20,.f.)               && font 'width'
   *      this.fontheight(20,.f.)              && font 'height'
   *      this.fontitalic(.t.,.T.)             && display italics
   *
   *      The following is from the sample form: TEXT.WFM, the
   *      onopen event for a vertical text control (this is the one
   *      with the blue text, on the right)
   *
   *      *-- When last parameter is .T., a redraw occurs.
   *      this.BottomTop = .f.                 && draw from top-to-bottom
   *      this.fontname("Times New Roman",.f.) && set font
   *      this.drawtext("VERTICAL TEXT",.f.)   && display text in quotes
   *      this.fontunderline(.t.,.f.)          && underline it
   *      this.colortext("FF0000",.f.)         && color is blue
   *      this.fontwidth(20,.f.)               && font 'width'
   *      this.fontheight(20,.f.)              && font 'height'
   *      this.fontitalic(.t.,.T.)             && display italics
   *
   *-- Properties:
   *
   *--    BottomTop
   *        Logical, defaults to true -- this is the default --
   *        draw text from the Bottom to the Top. Setting this
   *        to false in the object's OnOpen, you can draw the
   *        text from the top to the bottom instead.
   *
   *-- Methods:
   *
   *--   ColorText( <cColor>, <lRedraw> )
   *       <cColor> is standard hexidecimal color string, which can
   *                be gotten from a combination of the GetColor()
   *                function, and then placing each of the three
   *                returned values inside a call to "itoh" (integer
   *                to hexidecimal), and combining them in order
   *                to get the string.
   *       <lRedraw> tells the paintbox control to redraw the whole
   *                paintbox image -- this should be done for the last
   *                method called ... use .F. to _not_ redraw, use
   *                .t. to redraw ... (see examples above)
   *
   *--   DrawText( <cText>, <lRedraw> )
   *       <cText>  Text you wish to display
   *
   *--   FontHeight( <nHeight>, <lRedraw> )
   *       <nHeight> Font size in points, the height and width are
   *                 independant here, so you can actually give a 
   *                 height for a font that is different than the 
   *                 width -- this may give some odd results. If you 
   *                 want to use a specific font size and not distort
   *                 it, you should set FontHeight and FontWidth
   *                 to the same values ...
   *
   *--   FontWeight( <nWeight>, <lRedraw> )
   *       <nWeight> defaults to '700' -- change this to make font
   *                 more bold (make the value larger), to make it less
   *                 bold (make the value smaller) ...
   *
   *--   FontWidth( <nWidth>, <lRedraw> )
   *       <nWidth>  See FontHeight above ...
   *
   *--   FontItalic( <lItalic>, <lRedraw> )
   *       <lItalic> Logical -- .t. means to display italic ...
   *
   *--   FontName( <cFontName>, <lRedraw> )
   *       <cFontName> Font name as it appears in standard Getfont()
   *                 dialogs ...
   *
   *--   FontStrikeOut( <lStrikeout>, <lRedraw> )
   *       <lStrikeout> Logical -- .t. means to display with strikeout
   *
   *--   FontUnderline( <lUnderline>, <lRedraw> )
   *       <lUnderline> Logical -- .t. means to display underline ...
   *-------------------------------------------------------------------

   *-- Custom property to set direction of text, default is true,
   *-- as text is drawn bottom-to-top -- if you want to go 
   *-- top-to-bottom, change this to false in the _instance_ of the
   *-- class:
   this.BottomTop     = .t.

   *-- Standard properties --------------------------------------------
   this.ColorNormal   = "N/" + this.Parent.ColorNormal
   this.TabStop       = .F.    && not an interactive control
   this.Enabled       = .T.
   this.panColText    = 0           && ColorText
   this.pacFName      = "Arial"     && FontName
   this.panFHeight    = 8           && FontHeight
   this.panFWidth     = 8           && FontWidth
   this.panFWeight    = 700         && FontWeight
   this.panFRotation  = 0           && FontRotation
   this.palFItalic    = .F.         && FontItalic
   this.palFUnderline = .F.         && FontUnderline
   this.palFStrikeout = .F.         && FontStrikeout
   this.pnPPUX        = 0
   this.pnPPUY        = 0
   this.Text          = ""          && Text

   * Initialize data structures and internal utility properties
   this.xPaint =        Space(28)   && Buffer for PAINTSTRUCT structure
   this.xText =         Space(8)    && Buffer for DrawText RECT
 
   *-- define some inlines for the WINAPI stuff -----------------------
   #ifndef STORE8
      #define store8(nVal) (chr(bitand(nVal, 255)))
   #endif
   #ifndef STORE16
      #define store16(nVal) store8(nval) + store8(bitrshift(nVal, 8))
   #endif
   #ifndef MAKE8
	#define	MAKE8(cBuff, nIndex) Asc(SubStr(cBuff, nIndex))
   #endif
   #ifndef MAKEU16
      #define MAKEU16(cBuff, nIndex) MAKE8(cBuff, nIndex) +;
              BitLShift(MAKE8(cBuff, nIndex + 1), 8)
   #endif

   *-- define constants -----------------------------------------------
   #define   GCR_WIDTH           5
   #define   GCR_HEIGHT          7
   #define   OPAQUE              2
   #define   TRANSPARENT         1      
   #define   OUT_DEFAULT_PRECIS  0
   #define   CLIP_DEFAULT_PRECIS 0
   #define   DEFAULT_QUALITY     0
   #define   DEFAULT_PITCH       0
   #define   FF_DONTCARE         0
   #define   ANSI_CHARSET        0
   #define   OEM_CHARSET         HTOI("FF")
   #define   DT_BOTTOM           8
   #define   DT_CENTER           1
   #define   DT_EXPANDTABS       HToI("40")
   #define   DT_LEFT             0
   #define   DT_RIGHT            2
   #define   DT_SINGLELINE       HToI("20")
   #define   DT_TOP              0
   #define   DT_VCENTER          4
   #define   DT_WORDBREAK        HToI("10")
   #define   DT_CALCRECT         HToI("0400")
   #define   DT_NOCLIP           htoi("0100")  && used in top-to-bottom
   
   *-- Prototype Windows API functions --------------------------------
   if type("CreateFont") # "FP"
      extern CHANDLE CreateFont(CINT, CINT, CINT, CINT, CINT,;
                                CLOGICAL, CLOGICAL, CLOGICAL,;
                                CINT, CINT, CINT, CINT, CINT, CPTR) GDI
   endif
   if type("DrawText") # "FP"
      extern CINT DrawText(CHANDLE, CPTR, CINT, CPTR, CINT) User
   endif
   if type("SetBkColor") # "FP"
      extern CLONG SetBkColor(CHANDLE, CLONG) GDI
   endif
   if type("SetBkMode") # "FP"
      extern   CINT SetBkMode(CHANDLE, CINT) GDI
   endif
   if type("SetTextColor") # "FP"
      extern CLONG SetTextColor(CHANDLE, CLONG) GDI
   endif
   if type("BeginPaint") # "FP"
      extern CHANDLE beginpaint(CHANDLE, CPTR) user
   endif && type("BeginPaint") # "FP"
   if type("EndPaint") # "FP"
      extern CVOID endpaint(CHANDLE, CPTR) user
   endif && type("EndPaint") # "FP"
   if type("GetClientRect") # "FP"
      extern   CVOID   getclientrect(CHANDLE, CPTR) user
   endif && type("GetClientRect") # "FP"
   if type("InvalidateRectCli") # "FP"
      extern CVOID invalidaterectcli(CHANDLE, CLONG, CLOGICAL) user ;
                   from "InvalidateRect"
   endif && type("InvalidateRectCli") # "FP"
   if type("SelectObject") # "FP"
      extern CHANDLE   selectobject(CHANDLE, CHANDLE) GDI
   endif && type("SelectObject") # "FP"
   if type("UpdateWindow") # "FP"
      extern CVOID updatewindow(CHANDLE) user
   endif && type("UpdateWindow") # "FP"
   if type("DeleteObject") # "FP"
      extern CLOGICAL deleteobject(CHANDLE) GDI
   endif && type("DeleteObject") # "FP"
   
   PROCEDURE OnPaint
      *-- Suggestion from Romain to just have two versions of onPaint
      *-- routine ... turns out that it must be called in this fashion.
      *-- This over-ridden event sets the panFIncline value, and 
      *-- handles calling the appropriate paint routine.
      if this.BottomTop
         this.panFIncline = 900      && FontIncline
         class::PaintBottomTop()
      else
         this.panFIncline = 2700    && FontIncline
         class::PaintTopBottom()
      endif
      
   PROCEDURE BuildSize
   *-- Set internal properties based on current size and scaling 
   *-- factors
      LOCAL xRect
      xRect = Space(8)                 && Allocate RECT structure
      GetClientRect(this.hWND, xRect)  && Get width & height in pixels
      *-- Extract width & height in pixels
      this.pnPixWidth = MAKEU16(xRect, GCR_WIDTH) - 1
      this.pnPixHeight = MAKEU16(xRect, GCR_HEIGHT) - 1
      this.pnPPUX = Round(this.pnPixWidth / this.Width, 0)
      this.pnPPUY = Round(this.pnPixHeight / this.Height, 0)
      this.FontHeight(this.panFHeight) && Force recalc of font scaling
      this.FontWidth(this.panFWidth)
      this.pnOrgWidth = this.Width     && Save original width & height
      this.pnOrgHeight = this.Height
   
   *-- Redraw complete control ----------------------------------------
   PROCEDURE Redraw
      * Force redraw of object
      invalidaterectcli(this.hWND, 0, .f.)
      updatewindow(this.hWND)

   *-- Here the actual paint-job is done!------------------------------
   *-- Used to paint text from bottom to top (default)
   PROCEDURE PaintBottomTop
      *--use a flag since Buildsize() must be run after opening 
      *--(we need the window handles) and we want to keep ONOPEN() 
      *--for the form to set the values
      if type("this.sizecalculated")="U" 
               this.buildsize()  
               this.sizecalculated=.t.
      endif
      
      local hDC
      * Force Windows to draw entire object
      invalidaterectcli(this.hWND, 0, .f.)
      hDC = beginpaint(this.hWND, this.xPaint)
      
      if len(this.text) > 0
         SetBkMode(hDC, transparent) && Ensure font is drawn 
                                     && transparent
                      
         hDefFont = SelectObject(hDC, CreateFont( ;
                     this.pnAdjFHeight,;            && Height
                     this.pnAdjFWidth,;             && Width
                     this.panFIncline,;             && Incline
                     this.panFRotation,;            && Rotation
                     this.panFWeight,;              && Font weight
                     this.palFItalic,;              && Italic
                     this.palFUnderline,;           && Underline
                     this.palFStrikeout,;           && Strikeout
                     ANSI_CHARSET,;                 && Character set
                     OUT_DEFAULT_PRECIS,;           && Output precision
                     CLIP_DEFAULT_PRECIS,;          && Clipping precision
                     DEFAULT_QUALITY,;              && Output quality
                     DEFAULT_PITCH + FF_DONTCARE,;  && Pitch and Family
                     this.pacFName + Chr(0)  ))     && Font name

         SetTextColor(hDC, this.panColText)
         this.xtext = STORE16(0) +;  && Left side of rectangle
         STORE16(0) +;               && Top side of rectangle
         STORE16(this.pnpixwidth) +; && Right side of rectangle
         STORE16(this.pnpixheight+15)&& Bottom side of rectangle

         DrawText(hDC, this.text + Chr(0), -1, this.xtext,;
                                  DT_SINGLELINE + DT_BOTTOM + DT_LEFT)
         DeleteObject(SelectObject(hDC, hDefFont))
      endif
      EndPaint(this.hWND, this.xPaint) && EndPaint 'paired' with 
                                       && BeginPaint

   *-- used to paint from top to bottom, rather than bottom to top.
   PROCEDURE PaintTopBottom
      *-- use a flag since Buildsize() must be run after opening 
      *-- (we need the window handles) and we want to keep ONOPEN() 
      *-- for the form to set the values
      if type("this.sizecalculated")="U" 
         this.buildsize()  
         this.sizecalculated=.t.
      endif

      local hDC
      *-- Force Windows to draw entire object
      invalidaterectcli(this.hWND, 0, .f.)
      hDC = beginpaint(this.hWND, this.xPaint)

      if Len(this.text) > 0
         SetBkMode(hDC, transparent) && Ensure font is drawn 
                                     && transparent
         hDefFont = SelectObject(hDC, CreateFont( ;
                    this.pnAdjFHeight,;           && Height
                    this.pnAdjFWidth,;            && Width
                    this.panFIncline,;            && Incline
                    this.panFRotation,;           && Rotation
                    this.panFWeight,;             && Font weight
                    this.palFItalic,;             && Italic
                    this.palFUnderline,;          && Underline
                    this.palFStrikeout,;          && Strikeout
                    ANSI_CHARSET,;                && Character set
                    OUT_DEFAULT_PRECIS,;          && Output precision
                    CLIP_DEFAULT_PRECIS,;         && Clipping precision
                    DEFAULT_QUALITY,;             && Output quality
                    DEFAULT_PITCH + FF_DONTCARE,; && Pitch and Family
                    this.pacFName + Chr(0)  ))    && Font name
                     

         SetTextColor(hDC, this.panColText)
                                         && Left side of rectangle
         this.xtext = STORE16(0+(this.panfheight*2)) +;  
         STORE16(0) +;                && Top side of rectangle
         STORE16(this.pnpixwidth) +;  && Right side of rectangle
         STORE16(this.pnpixheight+15) && Bottom side of rectangle

         DrawText(hDC, this.text + Chr(0), -1, this.xtext,;
                                  DT_SINGLELINE + DT_TOP + DT_LEFT+;
                                  DT_NOCLIP)
         
         DeleteObject(SelectObject(hDC, hDefFont))
      endif
      EndPaint(this.hWND, this.xPaint) && EndPaint 'paired' with 
                                       && BeginPaint
   
   FUNCTION ColorText(cVal, lDraw)
      if PCount() > 0
         if Upper(Left(cVal, 2)) == "0X"
            cVal = Right(cVal, Len(cVal) - 2)
         endif
         this.panColText = HToI(cVal)  && Set internal ColorText
         if lDraw
            this.Redraw()
         endif
      endif
   RETURN IToH(this.panColText)

   FUNCTION FontHeight(nVal, lDraw)
      if (PCount() > 0) .AND. (nVal => 0)
         this.panFHeight = Int(nVal)   && Set internal FontHeight
         this.pnAdjFHeight = Round(this.panFHeight *;
                                  (this.pnPPUY / 10), 0) + 1
         if lDraw
            this.Redraw()
         endif
      endif
   RETURN this.panFHeight

   FUNCTION FontItalic(lVal, lDraw)
      if PCount() > 0
         if lVal; endif          && Force logical type detection
         this.palFItalic = lVal  && Set internal FontItalic
         if lDraw
            this.Redraw()
         endif
      endif
   RETURN this.palFItalic

   FUNCTION FontName(cVal, lDraw)
      if PCount() > 0
         this.pacFName = cVal     && Set internal FontName
         if lDraw
            this.Redraw()
         endif
      endif
   RETURN this.pacFName

   FUNCTION FontStrikeout(lVal, lDraw)
      if PCount() > 0
         if lVal; endif             && Force logical type detection
         this.palFStrikeout = lVal  && Set internal FontStrikeout
         if lDraw
            this.Redraw()
         endif
      endif
   RETURN this.palFStrikeout

   FUNCTION FontUnderline(lVal, lDraw)
      if PCount() > 0
         if lVal; endif             && Force logical type detection
         this.palFUnderline = lVal  && Set internal FontUnderline
         if lDraw
            this.Redraw()
         endif
      endif
   RETURN this.palFUnderline

   FUNCTION FontWeight(nVal, lDraw)
      if (PCount() > 0) .AND. (nVal => 0) .AND. (nVal =< 9)
         this.panFWeight = Int((nVal) * 100) && Set internal FontWeight
         if lDraw
            this.Redraw()
         endif
      endif
   RETURN this.panFWeight

   FUNCTION FontWidth(nVal, lDraw)
      if (PCount() > 0) .AND. (nVal => 0)
         this.panFWidth = Int(nVal)   && Set internal FontWidth
         this.pnAdjFWidth  = Round(this.panFWidth *;
                                  (this.pnPPUX / 10), 0) + 1
         if lDraw
            this.Redraw()
         endif
      endif
   RETURN this.panFWidth

   FUNCTION DrawText(cVal, lDraw)
      if (PCount() > 0) .AND. (Len(cVal) => 0)
         this.Text = cVal        && Set internal Text
         if lDraw
            this.Redraw()
         endif
      endif
   RETURN this.Text
   
ENDCLASS
*-- End of Class: VerticalText

*----------------------------------------------------------------------
*-- End of File: TEXT.CC
*----------------------------------------------------------------------
